@@ -69,6 +69,9 @@ AbstractModelSpecifics* precisionFactory<float>(
     case ModelType::COX :
         model = deviceFactory<BreslowTiedCoxProportionalHazards<float>,float>(modelData, deviceType, deviceName);
         break;
+    case ModelType::FINE_GRAY:
+        model = deviceFactory<BreslowTiedFineGray<float>,float>(modelData, deviceType, deviceName);
+        break;
     default:
         break;
     }
@@ -113,6 +116,9 @@ AbstractModelSpecifics* precisionFactory<double>(
     case ModelType::COX :
         model = deviceFactory<BreslowTiedCoxProportionalHazards<double>,double>(modelData, deviceType, deviceName);
         break;
+    case ModelType::FINE_GRAY:
+        model = deviceFactory<BreslowTiedFineGray<double>,double>(modelData, deviceType, deviceName);
+        break;
     default:
         break;
     }
