@@ -16,38 +16,39 @@ class CompressedDataColumn; // forward reference
 
 namespace bsccs {
 
+template <typename RealType>
 class SparseIndexer {
 public:
-	SparseIndexer(CompressedDataMatrix& matrix) : dataMatrix(matrix) {}
+	SparseIndexer(CompressedDataMatrix<RealType>& matrix) : dataMatrix(matrix) {}
 	virtual ~SparseIndexer() {}
-	
 
-	CompressedDataColumn& getColumn(const IdType& covariate) {
+
+	CompressedDataColumn<RealType>& getColumn(const IdType& covariate) {
 		return dataMatrix.getColumn(sparseMap[covariate]);
 	}
-	
+
 	void addColumn(const IdType& covariate, FormatType type) {
 		const int index = dataMatrix.getNumberOfColumns();
 		sparseMap.insert(std::make_pair(covariate, index));
-		
+
 		dataMatrix.push_back(type);
-		
+
 		// Add numerical labels
 		dataMatrix.getColumn(index).add_label(covariate);
 	}
-	
+
 	bool hasColumn(IdType covariate) const {
 		return sparseMap.count(covariate) != 0;
-	}	
+	}
 
 	int getIndex(IdType covariate){
 		return sparseMap[covariate];
 	}
-		
+
 private:
-	CompressedDataMatrix& dataMatrix;
+	CompressedDataMatrix<RealType>& dataMatrix;
 //	int nCovariates;
-	std::map<IdType, int> sparseMap;	
+	std::map<IdType, int> sparseMap;
 };
 
 } // namespace
